# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class NewLinesTestCase(RuleTestCase):
    rule_id = 'new-lines'

    def test_disabled(self):
        conf = ('new-line-at-end-of-file: disable\n'
                'new-lines: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('\r', conf)
        self.check('\r\n', conf)
        self.check('---\ntext\n', conf)
        self.check('---\r\ntext\r\n', conf)

    def test_unix_type(self):
        conf = 'new-lines: {type: unix}'
        self.check('', conf)
        self.check('\n', conf)
        self.check('\r\n', conf, problem=(1, 1))
        self.check('---\ntext\n', conf)
        self.check('---\r\ntext\r\n', conf, problem=(1, 4))

    def test_dos_type(self):
        conf = 'new-lines: {type: dos}\n'
        self.check('', conf)
        self.check('\n', conf, problem=(1, 1))
        self.check('\r\n', conf)
        self.check('---\ntext\n', conf, problem=(1, 4))
        self.check('---\r\ntext\r\n', conf)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import string

import yaml

from yamllint.linter import LintProblem


def spaces_after(token, prev, next, min=-1, max=-1,
                 min_desc=None, max_desc=None):
    if next is not None and token.end_mark.line == next.start_mark.line:
        spaces = next.start_mark.pointer - token.end_mark.pointer
        if max != - 1 and spaces > max:
            return LintProblem(token.start_mark.line + 1,
                               next.start_mark.column, max_desc)
        elif min != - 1 and spaces < min:
            return LintProblem(token.start_mark.line + 1,
                               next.start_mark.column + 1, min_desc)


def spaces_before(token, prev, next, min=-1, max=-1,
                  min_desc=None, max_desc=None):
    if (prev is not None and prev.end_mark.line == token.start_mark.line and
            # Discard tokens (only scalars?) that end at the start of next line
            (prev.end_mark.pointer == 0 or
             prev.end_mark.buffer[prev.end_mark.pointer - 1] != '\n')):
        spaces = token.start_mark.pointer - prev.end_mark.pointer
        if max != - 1 and spaces > max:
            return LintProblem(token.start_mark.line + 1,
                               token.start_mark.column, max_desc)
        elif min != - 1 and spaces < min:
            return LintProblem(token.start_mark.line + 1,
                               token.start_mark.column + 1, min_desc)


class Comment(object):
    def __init__(self, line, column, buffer, pointer):
        self.line = line
        self.column = column
        self.buffer = buffer
        self.pointer = pointer

    def __repr__(self):
        end = self.buffer.find('\n', self.pointer)
        if end == -1:
            end = self.buffer.find('\0', self.pointer)
        if end != -1:
            return self.buffer[self.pointer:end]
        return self.buffer[self.pointer:]

    def __eq__(self, other):
        return (self.line == other.line and
                self.column == other.column and
                str(self) == str(other))


def get_line_indent(token):
    """Finds the indent of the line the token starts in."""
    start = token.start_mark.buffer.rfind('\n', 0,
                                          token.start_mark.pointer) + 1
    content = start
    while token.start_mark.buffer[content] == ' ':
        content += 1
    return content - start


def get_real_end_line(token):
    """Finds the line on which the token really ends.

    With pyyaml, scalar tokens often end on a next line.
    """
    end_line = token.end_mark.line + 1

    if not isinstance(token, yaml.ScalarToken):
        return end_line

    pos = token.end_mark.pointer - 1
    while (pos >= token.start_mark.pointer - 1 and
           token.end_mark.buffer[pos] in string.whitespace):
        if token.end_mark.buffer[pos] == '\n':
            end_line -= 1
        pos -= 1
    return end_line


def get_comments_between_tokens(token1, token2, skip_first_line=False):
    if token2 is None:
        buf = token1.end_mark.buffer[token1.end_mark.pointer:]
    elif (token1.end_mark.line == token2.start_mark.line and
          not isinstance(token1, yaml.StreamStartToken) and
          not isinstance(token2, yaml.StreamEndToken)):
        return
    else:
        buf = token1.end_mark.buffer[token1.end_mark.pointer:
                                     token2.start_mark.pointer]

    line_no = token1.end_mark.line + 1
    column_no = token1.end_mark.column + 1
    pointer = token1.end_mark.pointer

    for line in buf.split('\n'):
        if skip_first_line:
            skip_first_line = False
        else:
            pos = line.find('#')
            if pos != -1:
                yield Comment(line_no, column_no + pos,
                              token1.end_mark.buffer, pointer + pos)

        pointer += len(line) + 1
        line_no += 1
        column_no = 1


def is_explicit_key(token):
    # explicit key:
    #   ? key
    #   : v
    # or
    #   ?
    #     key
    #   : v
    return (token.start_mark.pointer < token.end_mark.pointer and
            token.start_mark.buffer[token.start_mark.pointer] == '?')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

from yamllint import config


class SimpleConfigTestCase(unittest.TestCase):
    def test_parse_config(self):
        new = config.YamlLintConfig('rules:\n'
                                    '  colons:\n'
                                    '    max-spaces-before: 0\n'
                                    '    max-spaces-after: 1\n')

        self.assertEqual(list(new.rules.keys()), ['colons'])
        self.assertEqual(new.rules['colons']['max-spaces-before'], 0)
        self.assertEqual(new.rules['colons']['max-spaces-after'], 1)

        self.assertEqual(len(new.enabled_rules()), 1)

    def test_unknown_rule(self):
        with self.assertRaisesRegexp(
                config.YamlLintConfigError,
                'invalid config: no such rule: "this-one-does-not-exist"'):
            config.YamlLintConfig('rules:\n'
                                  '  this-one-does-not-exist: {}\n')

    def test_missing_option(self):
        with self.assertRaisesRegexp(
                config.YamlLintConfigError,
                'invalid config: missing option "max-spaces-before" '
                'for rule "colons"'):
            config.YamlLintConfig('rules:\n'
                                  '  colons:\n'
                                  '    max-spaces-after: 1\n')

    def test_unknown_option(self):
        with self.assertRaisesRegexp(
                config.YamlLintConfigError,
                'invalid config: unknown option "abcdef" for rule "colons"'):
            config.YamlLintConfig('rules:\n'
                                  '  colons:\n'
                                  '    max-spaces-before: 0\n'
                                  '    max-spaces-after: 1\n'
                                  '    abcdef: yes\n')


class ExtendedConfigTestCase(unittest.TestCase):
    def test_extend_add_rule(self):
        old = config.YamlLintConfig('rules:\n'
                                    '  colons:\n'
                                    '    max-spaces-before: 0\n'
                                    '    max-spaces-after: 1\n')
        new = config.YamlLintConfig('rules:\n'
                                    '  hyphens:\n'
                                    '    max-spaces-after: 2\n')
        new.extend(old)

        self.assertEqual(sorted(new.rules.keys()), ['colons', 'hyphens'])
        self.assertEqual(new.rules['colons']['max-spaces-before'], 0)
        self.assertEqual(new.rules['colons']['max-spaces-after'], 1)
        self.assertEqual(new.rules['hyphens']['max-spaces-after'], 2)

        self.assertEqual(len(new.enabled_rules()), 2)

    def test_extend_remove_rule(self):
        old = config.YamlLintConfig('rules:\n'
                                    '  colons:\n'
                                    '    max-spaces-before: 0\n'
                                    '    max-spaces-after: 1\n'
                                    '  hyphens:\n'
                                    '    max-spaces-after: 2\n')
        new = config.YamlLintConfig('rules:\n'
                                    '  colons: disable\n')
        new.extend(old)

        self.assertEqual(sorted(new.rules.keys()), ['colons', 'hyphens'])
        self.assertEqual(new.rules['colons'], False)
        self.assertEqual(new.rules['hyphens']['max-spaces-after'], 2)

        self.assertEqual(len(new.enabled_rules()), 1)

    def test_extend_edit_rule(self):
        old = config.YamlLintConfig('rules:\n'
                                    '  colons:\n'
                                    '    max-spaces-before: 0\n'
                                    '    max-spaces-after: 1\n'
                                    '  hyphens:\n'
                                    '    max-spaces-after: 2\n')
        new = config.YamlLintConfig('rules:\n'
                                    '  colons:\n'
                                    '    max-spaces-before: 3\n'
                                    '    max-spaces-after: 4\n')
        new.extend(old)

        self.assertEqual(sorted(new.rules.keys()), ['colons', 'hyphens'])
        self.assertEqual(new.rules['colons']['max-spaces-before'], 3)
        self.assertEqual(new.rules['colons']['max-spaces-after'], 4)
        self.assertEqual(new.rules['hyphens']['max-spaces-after'], 2)

        self.assertEqual(len(new.enabled_rules()), 2)

    def test_extend_reenable_rule(self):
        old = config.YamlLintConfig('rules:\n'
                                    '  colons:\n'
                                    '    max-spaces-before: 0\n'
                                    '    max-spaces-after: 1\n'
                                    '  hyphens: disable\n')
        new = config.YamlLintConfig('rules:\n'
                                    '  hyphens:\n'
                                    '    max-spaces-after: 2\n')
        new.extend(old)

        self.assertEqual(sorted(new.rules.keys()), ['colons', 'hyphens'])
        self.assertEqual(new.rules['colons']['max-spaces-before'], 0)
        self.assertEqual(new.rules['colons']['max-spaces-after'], 1)
        self.assertEqual(new.rules['hyphens']['max-spaces-after'], 2)

        self.assertEqual(len(new.enabled_rules()), 2)


class ExtendedLibraryConfigTestCase(unittest.TestCase):
    def test_extend_config_disable_rule(self):
        old = config.YamlLintConfig('extends: default')
        new = config.YamlLintConfig('extends: default\n'
                                    'rules:\n'
                                    '  trailing-spaces: disable\n')

        old.rules['trailing-spaces'] = False

        self.assertEqual(sorted(new.rules.keys()), sorted(old.rules.keys()))
        for rule in new.rules:
            self.assertEqual(new.rules[rule], old.rules[rule])

    def test_extend_config_override_whole_rule(self):
        old = config.YamlLintConfig('extends: default')
        new = config.YamlLintConfig('extends: default\n'
                                    'rules:\n'
                                    '  empty-lines:\n'
                                    '    max: 42\n'
                                    '    max-start: 43\n'
                                    '    max-end: 44\n')

        old.rules['empty-lines']['max'] = 42
        old.rules['empty-lines']['max-start'] = 43
        old.rules['empty-lines']['max-end'] = 44

        self.assertEqual(sorted(new.rules.keys()), sorted(old.rules.keys()))
        for rule in new.rules:
            self.assertEqual(new.rules[rule], old.rules[rule])

    def test_extend_config_override_rule_partly(self):
        old = config.YamlLintConfig('extends: default')
        new = config.YamlLintConfig('extends: default\n'
                                    'rules:\n'
                                    '  empty-lines:\n'
                                    '    max-start: 42\n')

        old.rules['empty-lines']['max-start'] = 42

        self.assertEqual(sorted(new.rules.keys()), sorted(old.rules.keys()))
        for rule in new.rules:
            self.assertEqual(new.rules[rule], old.rules[rule])



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to control the number of spaces inside brackets (``[`` and
``]``).

.. rubric:: Options

* ``min-spaces-inside`` defines the minimal number of spaces required inside
  brackets.
* ``max-spaces-inside`` defines the maximal number of spaces allowed inside
  brackets.

.. rubric:: Examples

#. With ``brackets: {min-spaces-inside: 0, max-spaces-inside: 0}``

   the following code snippet would **PASS**:
   ::

    object: [1, 2, abc]

   the following code snippet would **FAIL**:
   ::

    object: [ 1, 2, abc ]

#. With ``brackets: {min-spaces-inside: 1, max-spaces-inside: 3}``

   the following code snippet would **PASS**:
   ::

    object: [ 1, 2, abc ]

   the following code snippet would **PASS**:
   ::

    object: [ 1, 2, abc   ]

   the following code snippet would **FAIL**:
   ::

    object: [    1, 2, abc   ]

   the following code snippet would **FAIL**:
   ::

    object: [1, 2, abc ]
"""


import yaml

from yamllint.rules.common import spaces_after, spaces_before


ID = 'brackets'
TYPE = 'token'
CONF = {'min-spaces-inside': int,
        'max-spaces-inside': int}


def check(conf, token, prev, next, nextnext, context):
    if isinstance(token, yaml.FlowSequenceStartToken):
        problem = spaces_after(token, prev, next,
                               min=conf['min-spaces-inside'],
                               max=conf['max-spaces-inside'],
                               min_desc='too few spaces inside brackets',
                               max_desc='too many spaces inside brackets')
        if problem is not None:
            yield problem

    elif (isinstance(token, yaml.FlowSequenceEndToken) and
            (prev is None or
             not isinstance(prev, yaml.FlowSequenceStartToken))):
        problem = spaces_before(token, prev, next,
                                min=conf['min-spaces-inside'],
                                max=conf['max-spaces-inside'],
                                min_desc='too few spaces inside brackets',
                                max_desc='too many spaces inside brackets')
        if problem is not None:
            yield problem



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import yaml

from yamllint import parser


class LintProblem(object):
    """Represents a linting problem found by yamllint."""
    def __init__(self, line, column, desc='<no description>', rule=None):
        #: Line on which the problem was found (starting at 1)
        self.line = line
        #: Column on which the problem was found (starting at 1)
        self.column = column
        #: Human-readable description of the problem
        self.desc = desc
        #: Identifier of the rule that detected the problem
        self.rule = rule
        self.level = None

    @property
    def message(self):
        if self.rule is not None:
            return '%s (%s)' % (self.desc, self.rule)
        return self.desc

    def __eq__(self, other):
        return (self.line == other.line and
                self.column == other.column and
                self.rule == other.rule)

    def __lt__(self, other):
        return (self.line < other.line or
                (self.line == other.line and self.column < other.column))

    def __repr__(self):
        return '%d:%d: %s' % (self.line, self.column, self.message)


def get_costemic_problems(buffer, conf):
    rules = conf.enabled_rules()

    # Split token rules from line rules
    token_rules = [r for r in rules if r.TYPE == 'token']
    line_rules = [r for r in rules if r.TYPE == 'line']

    context = {}
    for rule in token_rules:
        context[rule.ID] = {}

    for elem in parser.token_or_line_generator(buffer):
        if isinstance(elem, parser.Token):
            for rule in token_rules:
                rule_conf = conf.rules[rule.ID]
                for problem in rule.check(rule_conf,
                                          elem.curr, elem.prev, elem.next,
                                          elem.nextnext,
                                          context[rule.ID]):
                    problem.rule = rule.ID
                    problem.level = rule_conf['level']
                    yield problem
        elif isinstance(elem, parser.Line):
            for rule in line_rules:
                rule_conf = conf.rules[rule.ID]
                for problem in rule.check(rule_conf, elem):
                    problem.rule = rule.ID
                    problem.level = rule_conf['level']
                    yield problem


def get_syntax_error(buffer):
    try:
        list(yaml.parse(buffer, Loader=yaml.BaseLoader))
    except yaml.error.MarkedYAMLError as e:
        problem = LintProblem(e.problem_mark.line + 1,
                              e.problem_mark.column + 1,
                              'syntax error: ' + e.problem)
        problem.level = 'error'
        return problem


def _run(buffer, conf):
    # If the document contains a syntax error, save it and yield it at the
    # right line
    syntax_error = get_syntax_error(buffer)

    for problem in get_costemic_problems(buffer, conf):
        # Insert the syntax error (if any) at the right place...
        if (syntax_error and syntax_error.line <= problem.line and
                syntax_error.column <= problem.column):
            yield syntax_error

            # If there is already a yamllint error at the same place, discard
            # it as it is probably redundant (and maybe it's just a 'warning',
            # in which case the script won't even exit with a failure status).
            if (syntax_error.line == problem.line and
                    syntax_error.column == problem.column):
                syntax_error = None
                continue

            syntax_error = None

        yield problem

    if syntax_error:
        yield syntax_error


def run(input, conf):
    """Lints a YAML source.

    Returns a generator of LintProblem objects.

    :param input: buffer, string or stream to read from
    :param conf: yamllint configuration object
    """
    if type(input) == str:
        return _run(input, conf)
    elif hasattr(input, 'read'):  # Python 2's file or Python 3's io.IOBase
        # We need to have everything in memory to parse correctly
        content = input.read()
        return _run(content, conf)
    else:
        raise TypeError('input should be a string or a stream')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""A linter for YAML files.

yamllint does not only check for syntax validity, but for weirdnesses like key
repetition and cosmetic problems such as lines length, trailing spaces,
indentation, etc."""


APP_NAME = 'yamllint'
APP_VERSION = '0.7.2'
APP_DESCRIPTION = __doc__

__author__ = u'Adrien Vergé'
__copyright__ = u'Copyright 2016, Adrien Vergé'
__license__ = 'GPLv3'
__version__ = APP_VERSION



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to set a limit to lines length.

.. rubric:: Options

* ``max`` defines the maximal (inclusive) length of lines.
* ``allow-non-breakable-words`` is used to allow non breakable words (without
  spaces inside) to overflow the limit. This is useful for long URLs, for
  instance. Use ``yes`` to allow, ``no`` to forbid.

.. rubric:: Examples

#. With ``line-length: {max: 70}``

   the following code snippet would **PASS**:
   ::

    long sentence:
      Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do
      eiusmod tempor incididunt ut labore et dolore magna aliqua.

   the following code snippet would **FAIL**:
   ::

    long sentence:
      Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod
      tempor incididunt ut labore et dolore magna aliqua.

#. With ``line-length: {max: 60, allow-non-breakable-words: yes}``

   the following code snippet would **PASS**:
   ::

    this:
      is:
        - a:
            http://localhost/very/very/very/very/very/very/very/very/long/url

    # this comment is too long,
    # but hard to split:
    # http://localhost/another/very/very/very/very/very/very/very/very/long/url

   the following code snippet would **FAIL**:
   ::

    - this line is waaaaaaaaaaaaaay too long but could be easily splitted...

#. With ``line-length: {max: 60, allow-non-breakable-words: no}``

   the following code snippet would **FAIL**:
   ::

    this:
      is:
        - a:
            http://localhost/very/very/very/very/very/very/very/very/long/url
"""


from yamllint.linter import LintProblem


ID = 'line-length'
TYPE = 'line'
CONF = {'max': int,
        'allow-non-breakable-words': bool}


def check(conf, line):
    if line.end - line.start > conf['max']:
        if conf['allow-non-breakable-words']:
            start = line.start
            while start < line.end and line.buffer[start] == ' ':
                start += 1

            if start != line.end:
                if line.buffer[start] == '#':
                    start += 2

                if line.buffer.find(' ', start, line.end) == -1:
                    return

        yield LintProblem(line.line_no, conf['max'] + 1,
                          'line too long (%d > %d characters)' %
                          (line.end - line.start, conf['max']))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to require a new line character (``\\n``) at the end of files.

The POSIX standard `requires the last line to end with a new line character
<http://pubs.opengroup.org/onlinepubs/9699919799/basedefs/V1_chap03.html#tag_03_206>`_.
All UNIX tools expect a new line at the end of files. Most text editors use
this convention too.
"""


from yamllint.linter import LintProblem


ID = 'new-line-at-end-of-file'
TYPE = 'line'


def check(conf, line):
    if line.end == len(line.buffer) and line.end > line.start:
        yield LintProblem(line.line_no, line.end - line.start + 1,
                          'no new line character at the end of file')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class CommentsIndentationTestCase(RuleTestCase):
    rule_id = 'comments-indentation'

    def test_disable(self):
        conf = 'comments-indentation: disable'
        self.check('---\n'
                   ' # line 1\n'
                   '# line 2\n'
                   '  # line 3\n'
                   '  # line 4\n'
                   '\n'
                   'obj:\n'
                   ' # these\n'
                   '   # are\n'
                   '  # [good]\n'
                   '# bad\n'
                   '      # comments\n'
                   '  a: b\n'
                   '\n'
                   'obj1:\n'
                   '  a: 1\n'
                   '  # comments\n'
                   '\n'
                   'obj2:\n'
                   '  b: 2\n'
                   '\n'
                   '# empty\n'
                   '#\n'
                   '# comment\n'
                   '...\n', conf)

    def test_enabled(self):
        conf = 'comments-indentation: {}'
        self.check('---\n'
                   '# line 1\n'
                   '# line 2\n', conf)
        self.check('---\n'
                   ' # line 1\n'
                   '# line 2\n', conf, problem=(2, 2))
        self.check('---\n'
                   '  # line 1\n'
                   '  # line 2\n', conf, problem1=(2, 3), problem2=(3, 3))
        self.check('---\n'
                   'obj:\n'
                   '  # normal\n'
                   '  a: b\n', conf)
        self.check('---\n'
                   'obj:\n'
                   ' # bad\n'
                   '  a: b\n', conf, problem=(3, 2))
        self.check('---\n'
                   'obj:\n'
                   '# bad\n'
                   '  a: b\n', conf, problem=(3, 1))
        self.check('---\n'
                   'obj:\n'
                   '   # bad\n'
                   '  a: b\n', conf, problem=(3, 4))
        self.check('---\n'
                   'obj:\n'
                   ' # these\n'
                   '   # are\n'
                   '  # [good]\n'
                   '# bad\n'
                   '      # comments\n'
                   '  a: b\n', conf,
                   problem1=(3, 2), problem2=(4, 4),
                   problem3=(6, 1), problem4=(7, 7))
        self.check('---\n'
                   'obj1:\n'
                   '  a: 1\n'
                   '  # the following line is disabled\n'
                   '  # b: 2\n', conf)
        self.check('---\n'
                   'obj1:\n'
                   '  a: 1\n'
                   '  # b: 2\n'
                   '\n'
                   'obj2:\n'
                   '  b: 2\n', conf)
        self.check('---\n'
                   'obj1:\n'
                   '  a: 1\n'
                   '  # b: 2\n'
                   '# this object is useless\n'
                   'obj2: no\n', conf)
        self.check('---\n'
                   'obj1:\n'
                   '  a: 1\n'
                   '# this object is useless\n'
                   '  # b: 2\n'
                   'obj2: no\n', conf, problem=(5, 3))
        self.check('---\n'
                   'obj1:\n'
                   '  a: 1\n'
                   '  # comments\n'
                   '  b: 2\n', conf)
        self.check('---\n'
                   'my list for today:\n'
                   '  - todo 1\n'
                   '  - todo 2\n'
                   '  # commented for now\n'
                   '  # - todo 3\n'
                   '...\n', conf)

    def test_first_line(self):
        conf = 'comments-indentation: {}'
        self.check('# comment\n', conf)
        self.check('  # comment\n', conf, problem=(1, 3))

    def test_no_newline_at_end(self):
        conf = ('comments-indentation: {}\n'
                'new-line-at-end-of-file: disable\n')
        self.check('# comment', conf)
        self.check('  # comment', conf, problem=(1, 3))

    def test_empty_comment(self):
        conf = 'comments-indentation: {}'
        self.check('---\n'
                   '# hey\n'
                   '# normal\n'
                   '#\n', conf)
        self.check('---\n'
                   '# hey\n'
                   '# normal\n'
                   ' #\n', conf, problem=(4, 2))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class ColonTestCase(RuleTestCase):
    rule_id = 'brackets'

    def test_disabled(self):
        conf = 'brackets: disable'
        self.check('---\n'
                   'array1: []\n'
                   'array2: [ ]\n'
                   'array3: [   a, b]\n'
                   'array4: [a, b, c ]\n'
                   'array5: [a, b, c ]\n'
                   'array6: [  a, b, c ]\n'
                   'array7: [   a, b, c ]\n', conf)

    def test_min_spaces(self):
        conf = 'brackets: {max-spaces-inside: -1, min-spaces-inside: 0}'
        self.check('---\n'
                   'array: []\n', conf)

        conf = 'brackets: {max-spaces-inside: -1, min-spaces-inside: 1}'
        self.check('---\n'
                   'array: []\n', conf, problem=(2, 9))
        self.check('---\n'
                   'array: [ ]\n', conf)
        self.check('---\n'
                   'array: [a, b]\n', conf, problem1=(2, 9), problem2=(2, 13))
        self.check('---\n'
                   'array: [ a, b ]\n', conf)
        self.check('---\n'
                   'array: [\n'
                   '  a,\n'
                   '  b\n'
                   ']\n', conf)

        conf = 'brackets: {max-spaces-inside: -1, min-spaces-inside: 3}'
        self.check('---\n'
                   'array: [ a, b ]\n', conf,
                   problem1=(2, 10), problem2=(2, 15))
        self.check('---\n'
                   'array: [   a, b   ]\n', conf)

    def test_max_spaces(self):
        conf = 'brackets: {max-spaces-inside: 0, min-spaces-inside: -1}'
        self.check('---\n'
                   'array: []\n', conf)
        self.check('---\n'
                   'array: [ ]\n', conf, problem=(2, 9))
        self.check('---\n'
                   'array: [a, b]\n', conf)
        self.check('---\n'
                   'array: [ a, b ]\n', conf,
                   problem1=(2, 9), problem2=(2, 14))
        self.check('---\n'
                   'array: [   a, b   ]\n', conf,
                   problem1=(2, 11), problem2=(2, 18))
        self.check('---\n'
                   'array: [\n'
                   '  a,\n'
                   '  b\n'
                   ']\n', conf)

        conf = 'brackets: {max-spaces-inside: 3, min-spaces-inside: -1}'
        self.check('---\n'
                   'array: [   a, b   ]\n', conf)
        self.check('---\n'
                   'array: [    a, b     ]\n', conf,
                   problem1=(2, 12), problem2=(2, 21))

    def test_min_and_max_spaces(self):
        conf = 'brackets: {max-spaces-inside: 0, min-spaces-inside: 0}'
        self.check('---\n'
                   'array: []\n', conf)
        self.check('---\n'
                   'array: [ ]\n', conf, problem=(2, 9))
        self.check('---\n'
                   'array: [   a, b]\n', conf, problem=(2, 11))

        conf = 'brackets: {max-spaces-inside: 1, min-spaces-inside: 1}'
        self.check('---\n'
                   'array: [a, b, c ]\n', conf, problem=(2, 9))

        conf = 'brackets: {max-spaces-inside: 2, min-spaces-inside: 0}'
        self.check('---\n'
                   'array: [a, b, c ]\n', conf)
        self.check('---\n'
                   'array: [  a, b, c ]\n', conf)
        self.check('---\n'
                   'array: [   a, b, c ]\n', conf, problem=(2, 11))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to forbid trailing spaces at the end of lines.

.. rubric:: Examples

#. With ``trailing-spaces: {}``

   the following code snippet would **PASS**:
   ::

    this document doesn't contain
    any trailing
    spaces

   the following code snippet would **FAIL**:
   ::

    this document contains     """ """
    trailing spaces
    on lines 1 and 3         """ """
"""


import string

from yamllint.linter import LintProblem


ID = 'trailing-spaces'
TYPE = 'line'


def check(conf, line):
    if line.end == 0:
        return

    # YAML recognizes two white space characters: space and tab.
    # http://yaml.org/spec/1.2/spec.html#id2775170

    pos = line.end
    while line.buffer[pos - 1] in string.whitespace and pos > line.start:
        pos -= 1

    if pos != line.end and line.buffer[pos] in ' \t':
        yield LintProblem(line.line_no, pos - line.start + 1,
                          'trailing spaces')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to control the number of spaces before and after commas (``,``).

.. rubric:: Options

* ``max-spaces-before`` defines the maximal number of spaces allowed before
  commas (use ``-1`` to disable).
* ``min-spaces-after`` defines the minimal number of spaces required after
  commas.
* ``max-spaces-after`` defines the maximal number of spaces allowed after
  commas (use ``-1`` to disable).

.. rubric:: Examples

#. With ``commas: {max-spaces-before: 0}``

   the following code snippet would **PASS**:
   ::

    strange var:
      [10, 20, 30, {x: 1, y: 2}]

   the following code snippet would **FAIL**:
   ::

    strange var:
      [10, 20 , 30, {x: 1, y: 2}]

#. With ``commas: {max-spaces-before: 2}``

   the following code snippet would **PASS**:
   ::

    strange var:
      [10  , 20 , 30,  {x: 1  , y: 2}]

#. With ``commas: {max-spaces-before: -1}``

   the following code snippet would **PASS**:
   ::

    strange var:
      [10,
       20   , 30
       ,   {x: 1, y: 2}]

#. With ``commas: {min-spaces-after: 1, max-spaces-after: 1}``

   the following code snippet would **PASS**:
   ::

    strange var:
      [10, 20,30, {x: 1, y: 2}]

   the following code snippet would **FAIL**:
   ::

    strange var:
      [10, 20,30,   {x: 1,   y: 2}]

#. With ``commas: {min-spaces-after: 1, max-spaces-after: 3}``

   the following code snippet would **PASS**:
   ::

    strange var:
      [10, 20,  30,  {x: 1,   y: 2}]

#. With ``commas: {min-spaces-after: 0, max-spaces-after: 1}``

   the following code snippet would **PASS**:
   ::

    strange var:
      [10, 20,30, {x: 1, y: 2}]
"""


import yaml

from yamllint.linter import LintProblem
from yamllint.rules.common import spaces_after, spaces_before


ID = 'commas'
TYPE = 'token'
CONF = {'max-spaces-before': int,
        'min-spaces-after': int,
        'max-spaces-after': int}


def check(conf, token, prev, next, nextnext, context):
    if isinstance(token, yaml.FlowEntryToken):
        if (prev is not None and conf['max-spaces-before'] != -1 and
                prev.end_mark.line < token.start_mark.line):
            yield LintProblem(token.start_mark.line + 1,
                              max(1, token.start_mark.column),
                              'too many spaces before comma')
        else:
            problem = spaces_before(token, prev, next,
                                    max=conf['max-spaces-before'],
                                    max_desc='too many spaces before comma')
            if problem is not None:
                yield problem

        problem = spaces_after(token, prev, next,
                               min=conf['min-spaces-after'],
                               max=conf['max-spaces-after'],
                               min_desc='too few spaces after comma',
                               max_desc='too many spaces after comma')
        if problem is not None:
            yield problem



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import yaml

from yamllint.parser import (line_generator, token_generator,
                             token_or_line_generator, Line, Token)


class ParserTestCase(unittest.TestCase):
    def test_line_generator(self):
        e = list(line_generator(''))
        self.assertEqual(len(e), 1)
        self.assertEqual(e[0].line_no, 1)
        self.assertEqual(e[0].start, 0)
        self.assertEqual(e[0].end, 0)

        e = list(line_generator('\n'))
        self.assertEqual(len(e), 2)

        e = list(line_generator(' \n'))
        self.assertEqual(len(e), 2)
        self.assertEqual(e[0].line_no, 1)
        self.assertEqual(e[0].start, 0)
        self.assertEqual(e[0].end, 1)

        e = list(line_generator('\n\n'))
        self.assertEqual(len(e), 3)

        e = list(line_generator('---\n'
                                'this is line 1\n'
                                'line 2\n'
                                '\n'
                                '3\n'))
        self.assertEqual(len(e), 6)
        self.assertEqual(e[0].line_no, 1)
        self.assertEqual(e[0].content, '---')
        self.assertEqual(e[2].content, 'line 2')
        self.assertEqual(e[3].content, '')
        self.assertEqual(e[5].line_no, 6)

        e = list(line_generator('test with\n'
                                'no newline\n'
                                'at the end'))
        self.assertEqual(len(e), 3)
        self.assertEqual(e[2].line_no, 3)
        self.assertEqual(e[2].content, 'at the end')

    def test_token_generator(self):
        e = list(token_generator(''))
        self.assertEqual(len(e), 2)
        self.assertEqual(e[0].prev, None)
        self.assertIsInstance(e[0].curr, yaml.Token)
        self.assertIsInstance(e[0].next, yaml.Token)
        self.assertEqual(e[1].prev, e[0].curr)
        self.assertEqual(e[1].curr, e[0].next)
        self.assertEqual(e[1].next, None)

        e = list(token_generator('---\n'
                                 'k: v\n'))
        self.assertEqual(len(e), 9)
        self.assertIsInstance(e[3].curr, yaml.KeyToken)
        self.assertIsInstance(e[5].curr, yaml.ValueToken)

    def test_token_or_line_generator(self):
        e = list(token_or_line_generator('---\n'
                                         'k: v\n'))
        self.assertEqual(len(e), 12)
        self.assertIsInstance(e[0], Token)
        self.assertIsInstance(e[0].curr, yaml.StreamStartToken)
        self.assertIsInstance(e[1], Token)
        self.assertIsInstance(e[1].curr, yaml.DocumentStartToken)
        self.assertIsInstance(e[2], Line)
        self.assertIsInstance(e[3].curr, yaml.BlockMappingStartToken)
        self.assertIsInstance(e[4].curr, yaml.KeyToken)
        self.assertIsInstance(e[6].curr, yaml.ValueToken)
        self.assertIsInstance(e[8], Line)
        self.assertIsInstance(e[11], Line)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import yaml


class Line(object):
    def __init__(self, line_no, buffer, start, end):
        self.line_no = line_no
        self.start = start
        self.end = end
        self.buffer = buffer

    @property
    def content(self):
        return self.buffer[self.start:self.end]


class Token(object):
    def __init__(self, line_no, curr, prev, next, nextnext):
        self.line_no = line_no
        self.curr = curr
        self.prev = prev
        self.next = next
        self.nextnext = nextnext


def line_generator(buffer):
    line_no = 1
    cur = 0
    next = buffer.find('\n')
    while next != -1:
        yield Line(line_no, buffer, start=cur, end=next)
        cur = next + 1
        next = buffer.find('\n', cur)
        line_no += 1

    yield Line(line_no, buffer, start=cur, end=len(buffer))


def token_generator(buffer):
    yaml_loader = yaml.BaseLoader(buffer)

    try:
        prev = None
        curr = yaml_loader.get_token()
        while curr is not None:
            next = yaml_loader.get_token()
            nextnext = yaml_loader.peek_token()

            yield Token(curr.start_mark.line + 1, curr, prev, next, nextnext)

            prev = curr
            curr = next

    except yaml.scanner.ScannerError:
        pass


def token_or_line_generator(buffer):
    """Generator that mixes tokens and lines, ordering them by line number"""
    token_gen = token_generator(buffer)
    line_gen = line_generator(buffer)

    token = next(token_gen, None)
    line = next(line_gen, None)

    while token is not None or line is not None:
        if token is None or (line is not None and
                             token.line_no > line.line_no):
            yield line
            line = next(line_gen, None)
        else:
            yield token
            token = next(token_gen, None)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class LineLengthTestCase(RuleTestCase):
    rule_id = 'line-length'

    def test_disabled(self):
        conf = ('line-length: disable\n'
                'empty-lines: disable\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('---\n', conf)
        self.check(81 * 'a', conf)
        self.check('---\n' + 81 * 'a' + '\n', conf)
        self.check(1000 * 'b', conf)
        self.check('---\n' + 1000 * 'b' + '\n', conf)

    def test_default(self):
        conf = ('line-length: {max: 80}\n'
                'empty-lines: disable\n'
                'new-line-at-end-of-file: disable\n'
                'document-start: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('---\n', conf)
        self.check(80 * 'a', conf)
        self.check('---\n' + 80 * 'a' + '\n', conf)
        self.check(16 * 'aaaa ' + 'z', conf, problem=(1, 81))
        self.check('---\n' + 16 * 'aaaa ' + 'z' + '\n', conf, problem=(2, 81))
        self.check(1000 * 'word ' + 'end', conf, problem=(1, 81))
        self.check('---\n' + 1000 * 'word ' + 'end\n', conf, problem=(2, 81))

    def test_max_length_10(self):
        conf = ('line-length: {max: 10}\n'
                'new-line-at-end-of-file: disable\n')
        self.check('---\nABCD EFGHI', conf)
        self.check('---\nABCD EFGHIJ', conf, problem=(2, 11))
        self.check('---\nABCD EFGHIJ\n', conf, problem=(2, 11))

    def test_spaces(self):
        conf = ('line-length: {max: 80}\n'
                'new-line-at-end-of-file: disable\n'
                'trailing-spaces: disable\n')
        self.check('---\n' + 81 * ' ', conf, problem=(2, 81))
        self.check('---\n' + 81 * ' ' + '\n', conf, problem=(2, 81))

    def test_non_breakable_word(self):
        conf = 'line-length: {max: 20, allow-non-breakable-words: yes}'
        self.check('---\n' + 30 * 'A' + '\n', conf)
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        http://localhost/very/long/url\n'
                   '...\n', conf)
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        # http://localhost/very/long/url\n'
                   '        comment\n'
                   '...\n', conf)

        conf = 'line-length: {max: 20, allow-non-breakable-words: no}'
        self.check('---\n' + 30 * 'A' + '\n', conf, problem=(2, 21))
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        http://localhost/very/long/url\n'
                   '...\n', conf, problem=(5, 21))
        self.check('---\n'
                   'this:\n'
                   '  is:\n'
                   '    - a:\n'
                   '        # http://localhost/very/long/url\n'
                   '        comment\n'
                   '...\n', conf, problem=(5, 21))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class TrailingSpacesTestCase(RuleTestCase):
    rule_id = 'trailing-spaces'

    def test_disabled(self):
        conf = 'trailing-spaces: disable'
        self.check('', conf)
        self.check('\n', conf)
        self.check('    \n', conf)
        self.check('---\n'
                   'some: text \n', conf)

    def test_enabled(self):
        conf = 'trailing-spaces: {}'
        self.check('', conf)
        self.check('\n', conf)
        self.check('    \n', conf, problem=(1, 1))
        self.check('\t\t\t\n', conf, problem=(1, 1, 'syntax'))
        self.check('---\n'
                   'some: text \n', conf, problem=(2, 11))
        self.check('---\n'
                   'some: text\t\n', conf, problem=(2, 11, 'syntax'))

    def test_with_dos_new_lines(self):
        conf = ('trailing-spaces: {}\n'
                'new-lines: {type: dos}\n')
        self.check('---\r\n'
                   'some: text\r\n', conf)
        self.check('---\r\n'
                   'some: text \r\n', conf, problem=(2, 11))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class HyphenTestCase(RuleTestCase):
    rule_id = 'hyphens'

    def test_disabled(self):
        conf = 'hyphens: disable'
        self.check('---\n'
                   '- elem1\n'
                   '- elem2\n', conf)
        self.check('---\n'
                   '- elem1\n'
                   '-  elem2\n', conf)
        self.check('---\n'
                   '-  elem1\n'
                   '-  elem2\n', conf)
        self.check('---\n'
                   '-  elem1\n'
                   '- elem2\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  - elem1\n'
                   '  -  elem2\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  -  elem1\n'
                   '  -  elem2\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  subobject:\n'
                   '    - elem1\n'
                   '    -  elem2\n', conf)
        self.check('---\n'
                   'object:\n'
                   '  subobject:\n'
                   '    -  elem1\n'
                   '    -  elem2\n', conf)

    def test_enabled(self):
        conf = 'hyphens: {max-spaces-after: 1}'
        self.check('---\n'
                   '- elem1\n'
                   '- elem2\n', conf)
        self.check('---\n'
                   '- elem1\n'
                   '-  elem2\n', conf, problem=(3, 3))
        self.check('---\n'
                   '-  elem1\n'
                   '-  elem2\n', conf, problem1=(2, 3), problem2=(3, 3))
        self.check('---\n'
                   '-  elem1\n'
                   '- elem2\n', conf, problem=(2, 3))
        self.check('---\n'
                   'object:\n'
                   '  - elem1\n'
                   '  -  elem2\n', conf, problem=(4, 5))
        self.check('---\n'
                   'object:\n'
                   '  -  elem1\n'
                   '  -  elem2\n', conf, problem1=(3, 5), problem2=(4, 5))
        self.check('---\n'
                   'object:\n'
                   '  subobject:\n'
                   '    - elem1\n'
                   '    -  elem2\n', conf, problem=(5, 7))
        self.check('---\n'
                   'object:\n'
                   '  subobject:\n'
                   '    -  elem1\n'
                   '    -  elem2\n', conf, problem1=(4, 7), problem2=(5, 7))

    def test_max_3(self):
        conf = 'hyphens: {max-spaces-after: 3}'
        self.check('---\n'
                   '-   elem1\n'
                   '-   elem2\n', conf)
        self.check('---\n'
                   '-    elem1\n'
                   '-   elem2\n', conf, problem=(2, 5))
        self.check('---\n'
                   'a:\n'
                   '  b:\n'
                   '    -   elem1\n'
                   '    -   elem2\n', conf)
        self.check('---\n'
                   'a:\n'
                   '  b:\n'
                   '    -    elem1\n'
                   '    -    elem2\n', conf, problem1=(4, 9), problem2=(5, 9))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import print_function
import os.path
import sys

import argparse

from yamllint import APP_DESCRIPTION, APP_NAME, APP_VERSION
from yamllint.config import YamlLintConfig, YamlLintConfigError
from yamllint import linter


def find_files_recursively(items):
    for item in items:
        if os.path.isdir(item):
            for root, dirnames, filenames in os.walk(item):
                for filename in [f for f in filenames
                                 if f.endswith(('.yml', '.yaml'))]:
                    yield os.path.join(root, filename)
        else:
            yield item


class Format(object):
    @staticmethod
    def parsable(problem, filename):
        return ('%(file)s:%(line)s:%(column)s: [%(level)s] %(message)s' %
                {'file': filename,
                 'line': problem.line,
                 'column': problem.column,
                 'level': problem.level,
                 'message': problem.message})

    @staticmethod
    def standard(problem, filename):
        line = '  \033[2m%d:%d\033[0m' % (problem.line, problem.column)
        line += max(20 - len(line), 0) * ' '
        if problem.level == 'warning':
            line += '\033[33m%s\033[0m' % problem.level
        else:
            line += '\033[31m%s\033[0m' % problem.level
        line += max(38 - len(line), 0) * ' '
        line += problem.desc
        if problem.rule:
            line += '  \033[2m(%s)\033[0m' % problem.rule
        return line


def run(argv=None):
    parser = argparse.ArgumentParser(prog=APP_NAME,
                                     description=APP_DESCRIPTION)
    parser.add_argument('files', metavar='FILE_OR_DIR', nargs='+',
                        help='files to check')
    parser.add_argument('-c', '--config', dest='config_file', action='store',
                        help='path to a custom configuration')
    parser.add_argument('-f', '--format',
                        choices=('parsable', 'standard'), default='standard',
                        help='format for parsing output')
    parser.add_argument('-v', '--version', action='version',
                        version='%s %s' % (APP_NAME, APP_VERSION))

    # TODO: read from stdin when no filename?

    args = parser.parse_args(argv)

    try:
        if args.config_file is not None:
            conf = YamlLintConfig(file=args.config_file)
        elif os.path.isfile('.yamllint'):
            conf = YamlLintConfig(file='.yamllint')
        else:
            conf = YamlLintConfig('extends: default')
    except YamlLintConfigError as e:
        print(e, file=sys.stderr)
        sys.exit(-1)

    return_code = 0

    for file in find_files_recursively(args.files):
        try:
            first = True
            with open(file) as f:
                for problem in linter.run(f, conf):
                    if args.format == 'parsable':
                        print(Format.parsable(problem, file))
                    else:
                        if first:
                            print('\033[4m%s\033[0m' % file)
                            first = False

                        print(Format.standard(problem, file))

                    if return_code == 0 and problem.level == 'error':
                        return_code = 1

            if not first and args.format != 'parsable':
                print('')
        except EnvironmentError as e:
            print(e)
            return_code = -1

    sys.exit(return_code)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to force the type of new line characters.

.. rubric:: Options

* Set ``type`` to ``unix`` to use UNIX-typed new line characters (``\\n``), or
  ``dos`` to use DOS-typed new line characters (``\\r\\n``).
"""


from yamllint.linter import LintProblem


ID = 'new-lines'
TYPE = 'line'
CONF = {'type': ('unix', 'dos')}


def check(conf, line):
    if line.start == 0 and len(line.buffer) > line.end:
        if conf['type'] == 'dos':
            if line.buffer[line.end - 1:line.end + 1] != '\r\n':
                yield LintProblem(1, line.end - line.start + 1,
                                  'wrong new line character: expected \\r\\n')
        else:
            if line.end > 0 and line.buffer[line.end - 1] == '\r':
                yield LintProblem(1, line.end - line.start,
                                  'wrong new line character: expected \\n')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import yaml

from yamllint.rules.common import (Comment, get_line_indent,
                                   get_comments_between_tokens)


class CommonTestCase(unittest.TestCase):
    def test_get_line_indent(self):
        tokens = list(yaml.scan('a: 1\n'
                                'b:\n'
                                '  - c: [2, 3, {d: 4}]\n'))

        self.assertEqual(tokens[3].value, 'a')
        self.assertEqual(tokens[5].value, '1')
        self.assertEqual(tokens[7].value, 'b')
        self.assertEqual(tokens[13].value, 'c')
        self.assertEqual(tokens[16].value, '2')
        self.assertEqual(tokens[18].value, '3')
        self.assertEqual(tokens[22].value, 'd')
        self.assertEqual(tokens[24].value, '4')

        for i in (3, 5):
            self.assertEqual(get_line_indent(tokens[i]), 0)
        for i in (7,):
            self.assertEqual(get_line_indent(tokens[i]), 0)
        for i in (13, 16, 18, 22, 24):
            self.assertEqual(get_line_indent(tokens[i]), 2)

    def check_comments(self, buffer, *expected):
        yaml_loader = yaml.BaseLoader(buffer)

        comments = []

        next = yaml_loader.peek_token()
        while next is not None:
            curr = yaml_loader.get_token()
            next = yaml_loader.peek_token()
            for comment in get_comments_between_tokens(curr, next):
                comments.append(comment)

        self.assertEqual(comments, list(expected))

    def test_get_comments_between_tokens(self):
        self.check_comments('# comment\n',
                            Comment(1, 1, '# comment', 0))
        self.check_comments('---\n'
                            '# comment\n'
                            '...\n',
                            Comment(2, 1, '# comment', 0))
        self.check_comments('---\n'
                            '# no newline char',
                            Comment(2, 1, '# no newline char', 0))
        self.check_comments('# just comment',
                            Comment(1, 1, '# just comment', 0))
        self.check_comments('\n'
                            '   # indented comment\n',
                            Comment(2, 4, '# indented comment', 0))
        self.check_comments('\n'
                            '# trailing spaces    \n',
                            Comment(2, 1, '# trailing spaces    ', 0))
        self.check_comments('# comment one\n'
                            '\n'
                            'key: val  # key=val\n'
                            '\n'
                            '# this is\n'
                            '# a block     \n'
                            '# comment\n'
                            '\n'
                            'other:\n'
                            '  - foo  # equals\n'
                            '         # bar\n',
                            Comment(1, 1, '# comment one', 0),
                            Comment(3, 11, '# key=val', 0),
                            Comment(5, 1, '# this is', 0),
                            Comment(6, 1, '# a block     ', 0),
                            Comment(7, 1, '# comment', 0),
                            Comment(10, 10, '# equals', 0),
                            Comment(11, 10, '# bar', 0))



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Use this rule to control the position and formatting of comments.

.. rubric:: Options

* Use ``require-starting-space`` to require a space character right after the
  ``#``. Set to ``yes`` to enable, ``no`` to disable.
* ``min-spaces-from-content`` is used to visually separate inline comments from
  content. It defines the minimal required number of spaces between a comment
  and its preceding content.

.. rubric:: Examples

#. With ``comments: {require-starting-space: yes}``

   the following code snippet would **PASS**:
   ::

    # This sentence
    # is a block comment

   the following code snippet would **FAIL**:
   ::

    #This sentence
    #is a block comment

#. With ``comments: {min-spaces-from-content: 2}``

   the following code snippet would **PASS**:
   ::

    x = 2 ^ 127 - 1  # Mersenne prime number

   the following code snippet would **FAIL**:
   ::

    x = 2 ^ 127 - 1 # Mersenne prime number
"""


import yaml

from yamllint.linter import LintProblem
from yamllint.rules.common import get_comments_between_tokens


ID = 'comments'
TYPE = 'token'
CONF = {'require-starting-space': bool,
        'min-spaces-from-content': int}


def check(conf, token, prev, next, nextnext, context):
    for comment in get_comments_between_tokens(token, next):
        if (conf['min-spaces-from-content'] != -1 and
                not isinstance(token, yaml.StreamStartToken) and
                comment.line == token.end_mark.line + 1):
            # Sometimes token end marks are on the next line
            if token.end_mark.buffer[token.end_mark.pointer - 1] != '\n':
                if (comment.pointer - token.end_mark.pointer <
                        conf['min-spaces-from-content']):
                    yield LintProblem(comment.line, comment.column,
                                      'too few spaces before comment')

        if (conf['require-starting-space'] and
                comment.pointer + 1 < len(comment.buffer) and
                comment.buffer[comment.pointer + 1] != ' ' and
                comment.buffer[comment.pointer + 1] != '\n'):
            yield LintProblem(comment.line, comment.column + 1,
                              'missing starting space in comment')



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os.path

import yaml

import yamllint.rules


class YamlLintConfigError(Exception):
    pass


class YamlLintConfig(object):
    def __init__(self, content=None, file=None):
        assert (content is None) ^ (file is None)

        if file is not None:
            with open(file) as f:
                content = f.read()

        self.parse(content)
        self.validate()

    def enabled_rules(self):
        return [yamllint.rules.get(id) for id, val in self.rules.items()
                if val is not False]

    def extend(self, base_config):
        assert isinstance(base_config, YamlLintConfig)

        for rule in self.rules:
            if (type(self.rules[rule]) == dict and
                    rule in base_config.rules and
                    base_config.rules[rule] is not False):
                base_config.rules[rule].update(self.rules[rule])
            else:
                base_config.rules[rule] = self.rules[rule]

        self.rules = base_config.rules

    def parse(self, raw_content):
        try:
            conf = yaml.safe_load(raw_content)
        except Exception as e:
            raise YamlLintConfigError('invalid config: %s' % e)

        self.rules = conf.get('rules', {})

        # Does this conf override another conf that we need to load?
        if 'extends' in conf:
            path = get_extended_config_file(conf['extends'])
            base = YamlLintConfig(file=path)
            try:
                self.extend(base)
            except Exception as e:
                raise YamlLintConfigError('invalid config: %s' % e)

    def validate(self):
        for id in self.rules:
            try:
                rule = yamllint.rules.get(id)
            except Exception as e:
                raise YamlLintConfigError('invalid config: %s' % e)

            self.rules[id] = validate_rule_conf(rule, self.rules[id])


def validate_rule_conf(rule, conf):
    if conf is False or conf == 'disable':
        return False

    if type(conf) == dict:
        if 'level' not in conf:
            conf['level'] = 'error'
        elif conf['level'] not in ('error', 'warning'):
            raise YamlLintConfigError(
                'invalid config: level should be "error" or "warning"')

        options = getattr(rule, 'CONF', {})
        for optkey in conf:
            if optkey == 'level':
                continue
            if optkey not in options:
                raise YamlLintConfigError(
                    'invalid config: unknown option "%s" for rule "%s"' %
                    (optkey, rule.ID))
            if type(options[optkey]) == tuple:
                if conf[optkey] not in options[optkey]:
                    raise YamlLintConfigError(
                        'invalid config: option "%s" of "%s" should be in %s'
                        % (optkey, rule.ID, options[optkey]))
            else:
                if type(conf[optkey]) != options[optkey]:
                    raise YamlLintConfigError(
                        'invalid config: option "%s" of "%s" should be %s'
                        % (optkey, rule.ID, options[optkey].__name__))
        for optkey in options:
            if optkey not in conf:
                raise YamlLintConfigError(
                    'invalid config: missing option "%s" for rule "%s"' %
                    (optkey, rule.ID))
    else:
        raise YamlLintConfigError(('invalid config: rule "%s": should be '
                                   'either "disable" or a dict') % rule.ID)

    return conf


def get_extended_config_file(name):
    # Is it a standard conf shipped with yamllint...
    if '/' not in name:
        std_conf = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                                'conf', name + '.yml')

        if os.path.isfile(std_conf):
            return std_conf

    # or a custom conf on filesystem?
    return name
